$VM = Get-SCVirtualMachine -VMMServer "{{ vmm_server }}" -ID "{{ vm_id }}"
$Drive = $VM | Get-SCVirtualDiskDrive | where {$_.ID -eq "{{ drive_id }}"}

Remove-SCVirtualDiskDrive -VirtualDiskDrive $Drive -JobVariable "job" -RunAsynchronously | Out-Null

$Resultado = [PSCustomObject]@{
    Status = 'OK'
    Guid = $job.ID
}
ConvertTo-Json -Depth 3 $Resultado
