"""
General configuration file for the vmm_manager.
"""

# VMM custom fields
FIELD_GROUP = (
    'VMM_MANAGER_AGRUPAMENTO',
    'Resource group (script vmm_manager).'
)

FIELD_ID = (
    'VMM_MANAGER_ID',
    'Resource name in the inventory file (script vmm_manager).'
)

FIELD_IMAGE = (
    'VMM_MANAGER_IMAGEM',
    'Resource template (script vmm_manager).'
)

FIELD_REGION = (
    'VMM_MANAGER_REGIAO',
    'Resource region (script vmm_manager).'
)

FIELD_NETWORK_DEFAULT = (
    'VMM_MANAGER_REDE_PRINCIPAL',
    'Default network (script vmm_manager).'
)
